package Joystick "Examples that compares the response of three analog joysticks to a circularly varying test force"
  package Components "Package containing components for joystick models."
    model TensionSpringDamper "One-directional tension spring with damping"
      import Modelica.Mechanics.MultiBody.Types;
      extends Modelica.Mechanics.MultiBody.Interfaces.PartialTwoFrames;
      parameter Boolean animation = true "= true, if animation shall be enabled";
      parameter Boolean showMass = true "= true, if point mass shall be visualized as sphere if animation=true and m>0";
      parameter Modelica.Units.SI.TranslationalSpringConstant c(final min = 0) "Spring constant";
      parameter Real d(final unit = "N/(m/s)", final min = 0, start = 1) "Damping constant";
      parameter Modelica.Units.SI.Length s_unstretched = 0 "Unstretched spring length";
      parameter Modelica.Units.SI.Mass m(min = 0) = 0 "Spring mass located on the connetion line between the origin of frame_a and the origin of frame_b";
      parameter Real lengthFraction(min = 0, max = 1) = 0.5 "Location of spring mass with respect to frame_a as a fraction of the distance from frame_a to frame_b (=0: at frame_a; =1: at frame_b)";
      input Modelica.Units.SI.Distance width = world.defaultForceWidth " Width of spring" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
      input Modelica.Units.SI.Distance coilWidth = width / 10 " Width of spring coil" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
      parameter Integer numberOfWindings = 5 " Number of spring windings" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
      input Modelica.Mechanics.MultiBody.Types.Color color = Modelica.Mechanics.MultiBody.Types.Defaults.SpringColor " Color of spring" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
      input Modelica.Mechanics.MultiBody.Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
      input Modelica.Units.SI.Diameter massDiameter = max(0, (width - 2 * coilWidth) * 0.9) " Diameter of mass point sphere" annotation(Dialog(tab = "Animation", group = "if animation = true and showMass = true", enable = animation and showMass));
      input Modelica.Mechanics.MultiBody.Types.Color massColor = Modelica.Mechanics.MultiBody.Types.Defaults.BodyColor " Color of mass point" annotation(Dialog(tab = "Animation", group = "if animation = true and showMass = true", enable = animation and showMass));
      Modelica.Mechanics.MultiBody.Forces.LineForceWithMass lineForce(animateLine = animation, animateMass = showMass, m = m, lengthFraction = lengthFraction, lineShapeType = "spring", lineShapeHeight = coilWidth * 2, lineShapeWidth = width, lineShapeExtra = numberOfWindings, lineShapeColor = color, specularCoefficient = specularCoefficient, massDiameter = massDiameter, massColor = massColor) annotation(Placement(visible = true, transformation(origin = {-0.0, 0.0}, extent = {{-20.0, -20.0}, {20.0, 20.0}}, rotation = 0)));
      Modelica.Mechanics.Translational.Components.ElastoGap elastoGap(c = c, s_rel0 = -s_unstretched, d = d) annotation(Placement(visible = true, transformation(origin = {0, 70}, extent = {{-10, -10}, {10, 10}}, rotation = -180)));
    equation
      connect(lineForce.flange_b, elastoGap.flange_a) annotation(Line(visible = true, origin = {12, 41.36}, points = {{0, -21.36}, {0, -17.96}, {1, -17.96}, {1, 28.64}, {-2, 28.64}}, color = {0, 127, 0}));
      connect(lineForce.flange_a, elastoGap.flange_b) annotation(Line(visible = true, origin = {-12, 41.36}, points = {{0, -21.36}, {0, -17.96}, {-1, -17.96}, {-1, 28.64}, {2, 28.64}}, color = {0, 127, 0}));
      connect(lineForce.frame_b, frame_b) annotation(Line(visible = true, origin = {60, 0}, points = {{-40, 0}, {40, 0}}, color = {95, 95, 95}, thickness = 0.5));
      connect(lineForce.frame_a, frame_a) annotation(Line(visible = true, origin = {-60, 0}, points = {{40, 0}, {-40, 0}}, color = {95, 95, 95}, thickness = 0.5));
      annotation(Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Line(visible = true, origin = {-20, 20}, points = {{50, -70}, {80, -70}}), Line(visible = true, origin = {-30, 40}, points = {{-50, 0}, {-20, 0}, {-10, -20}, {10, 20}, {30, -20}, {50, 20}, {60, 0}, {90, 0}}), Text(visible = true, origin = {-0, 31}, extent = {{-130, 49}, {132, 109}}, textString = "%name", fontName = "Arial"), Ellipse(visible = true, fillPattern = FillPattern.Solid, extent = {{-8, -8}, {8, 8}}), Line(visible = true, origin = {160, 0}, points = {{-80, 40}, {-80, -50}}, thickness = 5), Line(visible = true, origin = {0, 20}, points = {{-80, -70}, {-36.639, -70}}), Rectangle(visible = true, origin = {3.495, 13.644}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, extent = {{-43.495, -83.644}, {31.785, -33.457}}), Line(visible = true, origin = {12, 20}, points = {{-52, -40}, {38, -40}}), Line(visible = true, origin = {17, 30}, points = {{-52, -100}, {33, -100}}), Line(visible = true, origin = {-20, 10}, points = {{80, 30}, {80, -60}}), Text(visible = true, origin = {8, -48}, extent = {{-141, -92}, {125, -51}}, textString = "c=%c", fontName = "Arial"), Line(visible = true, origin = {-136.272, 70}, points = {{36.272, -70}, {56.272, -70}}), Line(visible = true, origin = {43.728, 70}, points = {{36.272, -70}, {56.272, -70}}), Line(visible = true, points = {{-80, 40}, {-80, -50}}, thickness = 0.75)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics), Documentation(info = "<html><div>
<div>This component models a one-directional tension spring with damping.</div>
</div></html>"));
    end TensionSpringDamper;

    model RotatingRadialForce "Radially directed force with circularly varying direction"
      Modelica.Mechanics.MultiBody.Forces.WorldForce force(animation = false) annotation(Placement(visible = true, transformation(origin = {60.0, 0.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = 0)));
      parameter Real f(final unit = "N", start = 1) "Force";
      Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_b annotation(Placement(visible = true, transformation(origin = {148.091, 0}, extent = {{-16, -16}, {16, 16}}, rotation = 0), iconTransformation(origin = {100, 0}, extent = {{-16, -16}, {16, 16}}, rotation = 0)));
      Modelica.Blocks.Sources.ContinuousClock clock annotation(Placement(visible = true, transformation(origin = {-120, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Blocks.Math.Gain gain1(k = 1) annotation(Placement(visible = true, transformation(origin = {-80, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Blocks.Math.Cos cos annotation(Placement(visible = true, transformation(origin = {-40, 40}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Blocks.Math.Sin sin annotation(Placement(visible = true, transformation(origin = {-40, -40}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Blocks.Sources.Constant const(k = 0) annotation(Placement(visible = true, transformation(origin = {0.0, 0.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = 0)));
      Modelica.Blocks.Math.Gain gain2(k = f) annotation(Placement(visible = true, transformation(origin = {0.0, 40.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = 0)));
      Modelica.Blocks.Math.Gain gain3(k = f) annotation(Placement(visible = true, transformation(origin = {0.0, -40.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = 0)));
    equation
      connect(sin.y, gain3.u) annotation(Line(visible = true, origin = {-20.5, -40}, points = {{-8.5, 0}, {8.5, 0}}, color = {1, 37, 163}));
      connect(gain3.y, force.force[3]) annotation(Line(visible = true, origin = {32.25, -20}, points = {{-21.25, -20}, {-2.25, -20}, {-2.25, 20}, {15.75, 20}}, color = {1, 37, 163}));
      connect(gain2.y, force.force[1]) annotation(Line(visible = true, origin = {32.25, 20}, points = {{-21.25, 20}, {-2.25, 20}, {-2.25, -20}, {15.75, -20}}, color = {1, 37, 163}));
      connect(cos.y, gain2.u) annotation(Line(visible = true, origin = {-20.5, 40}, points = {{-8.5, 0}, {8.5, 0}}, color = {1, 37, 163}));
      connect(gain1.y, cos.u) annotation(Line(visible = true, origin = {-57.75, 20}, points = {{-11.25, -20}, {2.75, -20}, {2.75, 20}, {5.75, 20}}, color = {1, 37, 163}));
      connect(gain1.y, sin.u) annotation(Line(visible = true, origin = {-57.75, -20}, points = {{-11.25, 20}, {2.75, 20}, {2.75, -20}, {5.75, -20}}, color = {1, 37, 163}));
      connect(const.y, force.force[2]) annotation(Line(visible = true, origin = {34.5, 0}, points = {{-23.5, 0}, {13.5, 0}}, color = {1, 37, 163}));
      connect(clock.y, gain1.u) annotation(Line(visible = true, origin = {-100.5, 0}, points = {{-8.5, 0}, {8.5, 0}}, color = {1, 37, 163}));
      connect(force.frame_b, frame_b) annotation(Line(visible = true, origin = {109.045, 0}, points = {{-39.045, 0}, {39.045, 0}}, color = {95, 95, 95}));
      annotation(Documentation(info = "<html><div>
<div>This component models a radially directed test force with a circularly varying direction.</div>
</div></html>"), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Ellipse(visible = true, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineThickness = 1.5, extent = {{-80, -80}, {80, 80}}), Line(visible = true, origin = {15, 36.834}, points = {{-15, -36.834}, {15, 36.834}}, thickness = 1.5), Line(visible = true, origin = {26.667, 61.136}, points = {{-16.667, -1.136}, {3.333, 12.273}, {13.333, -11.136}}, thickness = 1.5), Line(visible = true, origin = {-8.399, 26.21}, points = {{20.591, 3.79}, {12.549, 6.992}, {5.286, 6.992}, {-1.601, 3.79}, {-8.98, -3.643}, {-11.601, -13.759}, {-16.244, -4.162}}, thickness = 1.5), Line(visible = true, origin = {-15.966, 15.175}, points = {{-4.034, -2.724}, {4.034, 2.724}}, thickness = 1.5)}), Diagram(coordinateSystem(extent = {{-148.5, -80}, {148.5, 80}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10})));
    end RotatingRadialForce;

    model JoystickJoint "Two degree-of-freedom joystick joint"
      parameter Modelica.Units.SI.RotationalDampingConstant d(final min = 0, start = 0) "Damping constant";
      Modelica.Mechanics.MultiBody.Joints.Revolute revolute1(useAxisFlange = true, n = {1, 0, 0}, phi.start = initialxangle, phi.fixed = true) annotation(Placement(visible = true, transformation(origin = {-50.0, 0.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = 0)));
      Modelica.Mechanics.MultiBody.Joints.Revolute revolute2(n = {0, 0, 1}, useAxisFlange = true, phi.start = initialyangle, phi.fixed = true) annotation(Placement(visible = true, transformation(origin = {-0.0, 60.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = -270)));
      Modelica.Mechanics.Rotational.Components.Damper damper1(d = d) annotation(Placement(visible = true, transformation(origin = {-50.0, 30.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Damper damper2(d = d) annotation(Placement(visible = true, transformation(origin = {-30.0, 60.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = -270)));
      Modelica.Mechanics.Rotational.Interfaces.Flange_a flange_a1 annotation(Placement(visible = true, transformation(origin = {-148.112, -60}, extent = {{-10, -10}, {10, 10}}, rotation = 0), iconTransformation(origin = {-100, -45.394}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Interfaces.Flange_a flange_a2 annotation(Placement(visible = true, transformation(origin = {-70.0, 105.1826}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = 0), iconTransformation(origin = {-47.21, 100.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = 0)));
      Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_b annotation(Placement(visible = true, transformation(origin = {148.363, 0}, extent = {{-16, -16}, {16, 16}}, rotation = 0), iconTransformation(origin = {101.01, 0}, extent = {{-16, -16}, {16, 16}}, rotation = 0)));
      Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a annotation(Placement(visible = true, transformation(origin = {-147.818, 0}, extent = {{-16, -16}, {16, 16}}, rotation = 0), iconTransformation(origin = {-100, -4}, extent = {{-16, -16}, {16, 16}}, rotation = 0)));
      parameter Modelica.Units.SI.Angle initialxangle = 0;
      parameter Modelica.Units.SI.Angle initialyangle = 0;
    equation
      connect(revolute2.axis, flange_a2) annotation(Line(visible = true, origin = {-7.24, 80.873}, points = {{-2.76, -20.873}, {-5.86, -20.873}, {-5.86, 9.127}, {-62.76, 9.127}, {-62.76, 24.31}}, color = {64, 64, 64}));
      connect(revolute1.axis, flange_a1) annotation(Line(visible = true, origin = {-75.445, -3.4}, points = {{25.445, 13.4}, {25.445, 16.5}, {55.445, 16.5}, {55.445, -26.6}, {55.445, -56.6}, {-44.555, -56.6}, {-72.667, -56.6}}, color = {64, 64, 64}));
      connect(revolute1.frame_a, frame_a) annotation(Line(visible = true, origin = {-103.909, 0}, points = {{43.909, 0}, {-43.909, 0}}, color = {95, 95, 95}));
      connect(revolute2.frame_b, frame_b) annotation(Line(visible = true, origin = {42, 46}, points = {{-42, 24}, {-42, 34}, {-12, 34}, {-12, -46}, {106.363, -46}}, color = {95, 95, 95}));
      connect(revolute1.frame_b, revolute2.frame_a) annotation(Line(visible = true, origin = {-13.333, 16.667}, points = {{-26.667, -16.667}, {13.333, -16.667}, {13.333, 33.333}}, color = {95, 95, 95}));
      connect(revolute1.axis, damper1.flange_b) annotation(Line(visible = true, origin = {-42.8, 19.24}, points = {{-7.2, -9.24}, {-7.2, -6.14}, {5.8, -6.14}, {5.8, 10.76}, {2.8, 10.76}}, color = {64, 64, 64}));
      connect(damper1.flange_a, revolute1.support) annotation(Line(visible = true, origin = {-59.6, 19.24}, points = {{-0.4, 10.76}, {-3.4, 10.76}, {-3.4, -6.14}, {3.6, -6.14}, {3.6, -9.24}}, color = {64, 64, 64}));
      connect(damper2.flange_b, revolute2.axis) annotation(Line(visible = true, origin = {-19.24, 67.2}, points = {{-10.76, 2.8}, {-10.76, 5.8}, {6.14, 5.8}, {6.14, -7.2}, {9.24, -7.2}}, color = {64, 64, 64}));
      connect(damper2.flange_a, revolute2.support) annotation(Line(visible = true, origin = {-19.24, 50.4}, points = {{-10.76, -0.4}, {-10.76, -3.4}, {6.14, -3.4}, {6.14, 3.6}, {9.24, 3.6}}, color = {64, 64, 64}));
      annotation(Documentation(info = "<html><div>
<div>This component models a two-degree-of-freedom joystick joint.</div>
</div></html>"), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Rectangle(visible = true, origin = {0, 10}, fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, extent = {{30, -60}, {100, 60}}), Rectangle(visible = true, origin = {0, 10}, extent = {{30, -60}, {100, 60}}), Text(visible = true, origin = {9, 4}, textColor = {128, 128, 128}, extent = {{51, -14}, {87, 11}}, textString = "a", fontName = "Arial"), Rectangle(visible = true, origin = {-25.778, 0}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, extent = {{-54.222, -10}, {54.222, 11}}), Text(visible = true, origin = {0, -230}, extent = {{-150, 70}, {150, 130}}, textString = "%name", fontName = "Arial"), Rectangle(visible = true, origin = {-1.556, 25}, rotation = -90, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, extent = {{-55, -10}, {55, 11}})}), Diagram(coordinateSystem(extent = {{-148.5, -105.0}, {148.5, 105.0}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10})));
    end JoystickJoint;

    model ThreeSpringReturn "Joystick re-centering mechanism with three tension springs"
      parameter Modelica.Units.SI.TranslationalSpringConstant c(final min = 0) "Spring constant";
      parameter Real d(final unit = "N/(m/s)", final min = 0, start = 1) "Damping constant";
      parameter Modelica.Units.SI.Length spring_height(final min = 0, start = -0.02) "Height of tension springs fixed connections";
      parameter Modelica.Units.SI.Length s_unstretched(final min = 0, start = 0.02) "Unstretched length of each tension spring";
      Joystick.Components.TensionSpringDamper tensionSpring1(s_unstretched = s_unstretched, c = c, d = d, coilWidth = 0.0013) annotation(Placement(visible = true, transformation(origin = {-30.0, 0.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = 0)));
      Joystick.Components.TensionSpringDamper tensionSpring2(s_unstretched = s_unstretched, c = c, d = d, coilWidth = 0.0013) annotation(Placement(visible = true, transformation(origin = {20.0, -30.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = -270)));
      Joystick.Components.TensionSpringDamper tensionSpring3(s_unstretched = s_unstretched, c = c, d = d, coilWidth = 0.0013) annotation(Placement(visible = true, transformation(origin = {20.0, 30.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = -450)));
      Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_b annotation(Placement(visible = true, transformation(origin = {0, 0}, extent = {{-16, -16}, {16, 16}}, rotation = 0), iconTransformation(origin = {0, 0}, extent = {{-16, -16}, {16, 16}}, rotation = 0)));
      Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a annotation(Placement(visible = true, transformation(origin = {-150, 0}, extent = {{-16, -16}, {16, 16}}, rotation = 0), iconTransformation(origin = {-101.01, 0}, extent = {{-16, -16}, {16, 16}}, rotation = 0)));
      Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation1(r = {-1 * s_unstretched, 0, 0}) annotation(Placement(visible = true, transformation(origin = {-80.0, -0.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = 0)));
      Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation2(r = {s_unstretched * 0.5, 0, s_unstretched * 0.866}) annotation(Placement(visible = true, transformation(origin = {20.0, 70.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = -90)));
      Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation3(r = {s_unstretched * 0.5, 0, -s_unstretched * 0.866}) annotation(Placement(visible = true, transformation(origin = {20.0, -70.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = -270)));
    equation
      connect(fixedTranslation3.frame_a, frame_a) annotation(Line(visible = true, origin = {-74, -49.2}, points = {{94, -30.8}, {94, -33.8}, {-56, -33.8}, {-56, 49.2}, {-76, 49.2}}, color = {95, 95, 95}));
      connect(tensionSpring2.frame_a, fixedTranslation3.frame_b) annotation(Line(visible = true, origin = {20, -50}, points = {{0, 10}, {0, -10}}, color = {95, 95, 95}));
      connect(fixedTranslation2.frame_a, frame_a) annotation(Line(visible = true, origin = {-74, 52}, points = {{94, 28}, {94, 38}, {-56, 38}, {-56, -52}, {-76, -52}}, color = {95, 95, 95}));
      connect(tensionSpring3.frame_a, fixedTranslation2.frame_b) annotation(Line(visible = true, origin = {20, 50}, points = {{-0, -10}, {0, 10}}, color = {95, 95, 95}));
      connect(fixedTranslation1.frame_b, tensionSpring1.frame_a) annotation(Line(visible = true, origin = {-55, 0}, points = {{-15, 0}, {15, 0}}, color = {95, 95, 95}));
      connect(frame_a, fixedTranslation1.frame_a) annotation(Line(visible = true, origin = {-120, 0}, points = {{-30, 0}, {30, 0}}, color = {95, 95, 95}));
      connect(tensionSpring1.frame_b, frame_b) annotation(Line(visible = true, origin = {-10, 0}, points = {{-10, 0}, {10, 0}}, color = {95, 95, 95}));
      connect(tensionSpring3.frame_b, frame_b) annotation(Line(visible = true, origin = {10, 31.731}, points = {{10, -11.731}, {10, -31.731}, {-10, -31.731}, {-10, -31.731}}, color = {95, 95, 95}));
      connect(tensionSpring2.frame_b, frame_b) annotation(Line(visible = true, origin = {10, 21.731}, points = {{10, -41.731}, {10, -21.731}, {-10, -21.731}, {-10, -21.731}}, color = {95, 95, 95}));
      annotation(Documentation(info = "<html><div>
<div>This component models a joystick re-centering mechanism based on a symmetrical arrangement of three tension springs.</div>
</div></html>"), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Line(visible = true, origin = {-47.143, 2.857}, points = {{-32.857, -2.857}, {-22.857, -2.857}, {-12.857, 17.143}, {-2.857, -22.857}, {7.143, 17.143}, {17.143, -2.857}, {47.143, -2.857}}), Line(visible = true, origin = {45.714, 48.571}, points = {{-45.714, -48.571}, {-15.714, -18.571}, {-15.714, 1.429}, {14.286, -8.571}, {4.286, 21.429}, {24.286, 21.429}, {34.286, 31.429}}), Line(visible = true, origin = {48.571, -45.714}, rotation = -90, points = {{-45.714, -48.571}, {-15.714, -18.571}, {-15.714, 1.429}, {14.286, -8.571}, {4.286, 21.429}, {24.286, 21.429}, {34.286, 31.429}}), Line(visible = true, origin = {-90, 0}, points = {{0, 10}, {0, -10}}, thickness = 2), Line(visible = true, origin = {90, 90}, points = {{-10, 10}, {10, -10}}, thickness = 2), Line(visible = true, origin = {90, -90}, points = {{10, 10}, {-10, -10}}, thickness = 2), Line(visible = true, origin = {80, 80}, points = {{-10, 10}, {10, -10}}, thickness = 2), Line(visible = true, origin = {-80, 0}, points = {{0, 10}, {0, -10}}, thickness = 2), Line(visible = true, origin = {80, -80}, points = {{10, 10}, {-10, -10}}, thickness = 2), Text(visible = true, origin = {0, -230}, extent = {{-150, 70}, {150, 130}}, textString = "%name", fontName = "Arial")}), Diagram(coordinateSystem(extent = {{-148.5, -105.0}, {148.5, 105.0}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10})));
    end ThreeSpringReturn;

    model FourSpringReturn "Joystick re-centering mechanism with four tension springs"
      parameter Modelica.Units.SI.TranslationalSpringConstant c(final min = 0) "Spring constant";
      parameter Real d(final unit = "N/(m/s)", final min = 0, start = 1) "Damping constant";
      parameter Modelica.Units.SI.Length spring_height(final min = 0, start = -0.02) "Height of tension springs fixed connections";
      parameter Modelica.Units.SI.Length s_unstretched(final min = 0, start = 0.02) "Unstretched length of each tension spring";
      Joystick.Components.TensionSpringDamper tensionSpring1(s_unstretched = s_unstretched, c = c, d = d, coilWidth = 0.0013) annotation(Placement(visible = true, transformation(origin = {-30.0, 0.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = 0)));
      Joystick.Components.TensionSpringDamper tensionSpring2(s_unstretched = s_unstretched, c = c, d = d, coilWidth = 0.0013) annotation(Placement(visible = true, transformation(origin = {0.0, -30.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = -270)));
      Joystick.Components.TensionSpringDamper tensionSpring3(s_unstretched = s_unstretched, c = c, d = d, coilWidth = 0.0013) annotation(Placement(visible = true, transformation(origin = {0.0, 30.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = -450)));
      Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_b annotation(Placement(visible = true, transformation(origin = {0, 0}, extent = {{-16, -16}, {16, 16}}, rotation = 0), iconTransformation(origin = {0, 0}, extent = {{-16, -16}, {16, 16}}, rotation = 0)));
      Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a annotation(Placement(visible = true, transformation(origin = {-150, 0}, extent = {{-16, -16}, {16, 16}}, rotation = 0), iconTransformation(origin = {-101.01, 0}, extent = {{-16, -16}, {16, 16}}, rotation = 0)));
      Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation1(r = {-1 * s_unstretched, 0, 0}) annotation(Placement(visible = true, transformation(origin = {-80.0, -0.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = 0)));
      Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation2(r = {0, 0, s_unstretched * 1}) annotation(Placement(visible = true, transformation(origin = {0.0, 70.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = -90)));
      Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation3(r = {0, 0, -s_unstretched * 1}) annotation(Placement(visible = true, transformation(origin = {0.0, -70.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = -270)));
      Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation4(r = {1 * s_unstretched, 0, 0}) annotation(Placement(visible = true, transformation(origin = {80.0, -0.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = -180)));
      Joystick.Components.TensionSpringDamper tensionSpring4(c = c, d = d, s_unstretched = s_unstretched, coilWidth = 0.0013) annotation(Placement(visible = true, transformation(origin = {40.0, -0.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = -180)));
    equation
      connect(tensionSpring3.frame_a, fixedTranslation2.frame_b) annotation(Line(visible = true, origin = {0, 50}, points = {{-0, -10}, {0, 10}}, color = {95, 95, 95}));
      connect(tensionSpring3.frame_b, frame_b) annotation(Line(visible = true, origin = {10, 31.731}, points = {{-10, -11.731}, {-10, -31.731}, {-10, -31.731}, {-10, -31.731}}, color = {95, 95, 95}));
      connect(fixedTranslation2.frame_a, frame_a) annotation(Line(visible = true, origin = {-74, 52}, points = {{74, 28}, {74, 38}, {-56, 38}, {-56, -52}, {-76, -52}}, color = {95, 95, 95}));
      connect(tensionSpring4.frame_b, frame_b) annotation(Line(visible = true, origin = {15, 0}, points = {{15, 0}, {-15, 0}}, color = {95, 95, 95}));
      connect(fixedTranslation3.frame_a, fixedTranslation4.frame_a) annotation(Line(visible = true, origin = {62, -52}, points = {{-62, -28}, {-62, -38}, {48, -38}, {48, 52}, {28, 52}}, color = {95, 95, 95}));
      connect(fixedTranslation4.frame_b, tensionSpring4.frame_a) annotation(Line(visible = true, origin = {75, 0}, rotation = -180, points = {{5, 0}, {25, 0}}, color = {95, 95, 95}));
      connect(fixedTranslation3.frame_a, frame_a) annotation(Line(visible = true, origin = {-74, -49.2}, points = {{74, -30.8}, {74, -40.8}, {-56, -40.8}, {-56, 49.2}, {-76, 49.2}}, color = {95, 95, 95}));
      connect(tensionSpring2.frame_b, frame_b) annotation(Line(visible = true, origin = {10, 21.731}, points = {{-10, -41.731}, {-10, -21.731}, {-10, -21.731}, {-10, -21.731}}, color = {95, 95, 95}));
      connect(tensionSpring2.frame_a, fixedTranslation3.frame_b) annotation(Line(visible = true, origin = {0, -50}, points = {{0, 10}, {0, -10}}, color = {95, 95, 95}));
      connect(fixedTranslation1.frame_b, tensionSpring1.frame_a) annotation(Line(visible = true, origin = {-55, 0}, points = {{-15, 0}, {15, 0}}, color = {95, 95, 95}));
      connect(frame_a, fixedTranslation1.frame_a) annotation(Line(visible = true, origin = {-120, 0}, points = {{-30, 0}, {30, 0}}, color = {95, 95, 95}));
      connect(tensionSpring1.frame_b, frame_b) annotation(Line(visible = true, origin = {-10, 0}, points = {{-10, 0}, {10, 0}}, color = {95, 95, 95}));
      annotation(Documentation(info = "<html><div>
<div>This component models a joystick re-centering mechanism based on a symmetrical arrangement of four tension springs.</div>
</div></html>"), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Line(visible = true, origin = {-47.143, 2.857}, points = {{-32.857, -2.857}, {-22.857, -2.857}, {-12.857, 17.143}, {-2.857, -22.857}, {7.143, 17.143}, {17.143, -2.857}, {47.143, -2.857}}), Line(visible = true, origin = {-90, 0}, points = {{0, 10}, {0, -10}}, thickness = 2), Line(visible = true, origin = {-80, 0}, points = {{0, 10}, {0, -10}}, thickness = 2), Line(visible = true, origin = {2.857, 47.143}, rotation = -90, points = {{-32.857, -2.857}, {-22.857, -2.857}, {-12.857, 17.143}, {-2.857, -22.857}, {7.143, 17.143}, {17.143, -2.857}, {47.143, -2.857}}), Line(visible = true, origin = {0, 80}, rotation = -90, points = {{0, 10}, {0, -10}}, thickness = 2), Line(visible = true, origin = {0, 90}, rotation = -90, points = {{0, 10}, {0, -10}}, thickness = 2), Line(visible = true, origin = {0, -90}, rotation = -90, points = {{0, 10}, {0, -10}}, thickness = 2), Line(visible = true, origin = {0, -80}, rotation = -90, points = {{0, 10}, {0, -10}}, thickness = 2), Line(visible = true, origin = {-2.857, -47.143}, rotation = -270, points = {{-32.857, -2.857}, {-22.857, -2.857}, {-12.857, 17.143}, {-2.857, -22.857}, {7.143, 17.143}, {17.143, -2.857}, {47.143, -2.857}}), Line(visible = true, origin = {47.143, -2.857}, rotation = -180, points = {{-32.857, -2.857}, {-22.857, -2.857}, {-12.857, 17.143}, {-2.857, -22.857}, {7.143, 17.143}, {17.143, -2.857}, {47.143, -2.857}}), Line(visible = true, origin = {80, 0}, points = {{0, 10}, {0, -10}}, thickness = 2), Line(visible = true, origin = {90, 0}, points = {{0, 10}, {0, -10}}, thickness = 2), Text(visible = true, origin = {0, -230}, extent = {{-150, 70}, {150, 130}}, textString = "%name", fontName = "Arial")}), Diagram(coordinateSystem(extent = {{-148.5, -105.0}, {148.5, 105.0}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10})));
    end FourSpringReturn;

    model SixSpringReturn "Joystick re-centering mechanism with six tension springs"
      parameter Modelica.Units.SI.TranslationalSpringConstant c(final min = 0) "Spring constant";
      parameter Real d(final unit = "N/(m/s)", final min = 0, start = 1) "Damping constant";
      parameter Modelica.Units.SI.Length spring_height(final min = 0, start = -0.02) "Height of tension springs fixed connections";
      parameter Modelica.Units.SI.Length s_unstretched(final min = 0, start = 0.02) "Unstretched length of each tension spring";
      Joystick.Components.TensionSpringDamper tensionSpring1(s_unstretched = s_unstretched, c = c, d = d, coilWidth = 0.0013) annotation(Placement(visible = true, transformation(origin = {-60.0, 0.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = 0)));
      Joystick.Components.TensionSpringDamper tensionSpring2(s_unstretched = s_unstretched, c = c, d = d, coilWidth = 0.0013) annotation(Placement(visible = true, transformation(origin = {20.0, -30.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = -270)));
      Joystick.Components.TensionSpringDamper tensionSpring3(s_unstretched = s_unstretched, c = c, d = d, coilWidth = 0.0013) annotation(Placement(visible = true, transformation(origin = {20.0, 30.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = -450)));
      Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_b annotation(Placement(visible = true, transformation(origin = {0, 0}, extent = {{-16, -16}, {16, 16}}, rotation = 0), iconTransformation(origin = {0, 0}, extent = {{-16, -16}, {16, 16}}, rotation = 0)));
      Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a annotation(Placement(visible = true, transformation(origin = {-150, 0}, extent = {{-16, -16}, {16, 16}}, rotation = 0), iconTransformation(origin = {-101.01, 0}, extent = {{-16, -16}, {16, 16}}, rotation = 0)));
      Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation1(r = {-1 * s_unstretched, 0, 0}) annotation(Placement(visible = true, transformation(origin = {-100.0, 0.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = 0)));
      Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation2(r = {s_unstretched * 0.5, 0, s_unstretched * 0.866}) annotation(Placement(visible = true, transformation(origin = {20.0, 70.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = -90)));
      Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation3(r = {s_unstretched * 0.5, 0, -s_unstretched * 0.866}) annotation(Placement(visible = true, transformation(origin = {20.0, -70.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = -270)));
      Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation4(r = {-s_unstretched * 0.5, 0, -s_unstretched * 0.866}) annotation(Placement(visible = true, transformation(origin = {-20.0, -70.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = -270)));
      Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation5(r = {-s_unstretched * 0.5, 0, s_unstretched * 0.866}) annotation(Placement(visible = true, transformation(origin = {-20.0, 70.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = -90)));
      Joystick.Components.TensionSpringDamper tensionSpring4(c = c, d = d, s_unstretched = s_unstretched, coilWidth = 0.0013) annotation(Placement(visible = true, transformation(origin = {-20.0, 30.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = -450)));
      Joystick.Components.TensionSpringDamper tensionSpring5(c = c, d = d, s_unstretched = s_unstretched, coilWidth = 0.0013) annotation(Placement(visible = true, transformation(origin = {-20.0, -30.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = -270)));
      Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation6(r = {1 * s_unstretched, 0, 0}) annotation(Placement(visible = true, transformation(origin = {100.0, 0.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = -180)));
      Joystick.Components.TensionSpringDamper tensionSpring6(c = c, d = d, s_unstretched = s_unstretched, coilWidth = 0.0013) annotation(Placement(visible = true, transformation(origin = {60.0, 0.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = -180)));
    equation
      connect(fixedTranslation2.frame_a, frame_a) annotation(Line(visible = true, origin = {-74, 52}, points = {{94, 28}, {94, 38}, {-56, 38}, {-56, -52}, {-76, -52}}, color = {95, 95, 95}));
      connect(fixedTranslation5.frame_a, fixedTranslation2.frame_a) annotation(Line(visible = true, origin = {0, 85}, points = {{-20, -5}, {-20, 5}, {20, 5}, {20, -5}}, color = {95, 95, 95}));
      connect(fixedTranslation3.frame_a, frame_a) annotation(Line(visible = true, origin = {-74, -52}, points = {{94, -28}, {94, -38}, {-56, -38}, {-56, 52}, {-76, 52}}, color = {95, 95, 95}));
      connect(fixedTranslation4.frame_a, fixedTranslation3.frame_a) annotation(Line(visible = true, origin = {0, -85}, points = {{-20, 5}, {-20, -5}, {20, -5}, {20, 5}}, color = {95, 95, 95}));
      connect(fixedTranslation3.frame_a, fixedTranslation6.frame_a) annotation(Line(visible = true, origin = {82, -52}, points = {{-62, -28}, {-62, -38}, {48, -38}, {48, 52}, {28, 52}}, color = {95, 95, 95}));
      connect(tensionSpring4.frame_b, frame_b) annotation(Line(visible = true, origin = {-13.333, 6.667}, points = {{-6.667, 13.333}, {-6.667, -6.667}, {13.333, -6.667}}, color = {95, 95, 95}));
      connect(tensionSpring6.frame_b, frame_b) annotation(Line(visible = true, origin = {25, 0}, points = {{25, 0}, {-25, 0}}, color = {95, 95, 95}));
      connect(tensionSpring5.frame_b, frame_b) annotation(Line(visible = true, origin = {-13.333, -6.667}, points = {{-6.667, -13.333}, {-6.667, 6.667}, {13.333, 6.667}}, color = {95, 95, 95}));
      connect(tensionSpring2.frame_b, frame_b) annotation(Line(visible = true, origin = {10, 21.731}, points = {{10, -41.731}, {10, -21.731}, {-10, -21.731}, {-10, -21.731}}, color = {95, 95, 95}));
      connect(tensionSpring3.frame_b, frame_b) annotation(Line(visible = true, origin = {10, 31.731}, points = {{10, -11.731}, {10, -31.731}, {-10, -31.731}, {-10, -31.731}}, color = {95, 95, 95}));
      connect(tensionSpring1.frame_b, frame_b) annotation(Line(visible = true, origin = {-25, 0}, points = {{-25, 0}, {25, 0}}, color = {95, 95, 95}));
      connect(tensionSpring3.frame_a, fixedTranslation2.frame_b) annotation(Line(visible = true, origin = {20, 50}, points = {{-0, -10}, {0, 10}}, color = {95, 95, 95}));
      connect(tensionSpring2.frame_a, fixedTranslation3.frame_b) annotation(Line(visible = true, origin = {20, -50}, points = {{0, 10}, {0, -10}}, color = {95, 95, 95}));
      connect(fixedTranslation6.frame_b, tensionSpring6.frame_a) annotation(Line(visible = true, origin = {80, 0}, points = {{10, 0}, {-10, 0}}, color = {95, 95, 95}));
      connect(fixedTranslation1.frame_b, tensionSpring1.frame_a) annotation(Line(visible = true, origin = {-80, 0}, points = {{-10, 0}, {10, 0}}, color = {95, 95, 95}));
      connect(fixedTranslation5.frame_b, tensionSpring4.frame_a) annotation(Line(visible = true, origin = {-20, 50}, points = {{0, 10}, {-0, -10}}, color = {95, 95, 95}));
      connect(fixedTranslation4.frame_b, tensionSpring5.frame_a) annotation(Line(visible = true, origin = {-20, -50}, points = {{0, -10}, {0, 10}}, color = {95, 95, 95}));
      connect(frame_a, fixedTranslation1.frame_a) annotation(Line(visible = true, origin = {-130, 0}, points = {{-20, 0}, {20, 0}}, color = {95, 95, 95}));
      annotation(Documentation(info = "<html><div>
<div>This component models a joystick re-centering mechanism based on a symmetrical arrangement of six tension springs.</div>
</div></html>"), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Line(visible = true, origin = {-47.143, 2.857}, points = {{-32.857, -2.857}, {-22.857, -2.857}, {-12.857, 17.143}, {-2.857, -22.857}, {7.143, 17.143}, {17.143, -2.857}, {47.143, -2.857}}), Line(visible = true, origin = {45.714, 48.571}, points = {{-45.714, -48.571}, {-15.714, -18.571}, {-15.714, 1.429}, {14.286, -8.571}, {4.286, 21.429}, {24.286, 21.429}, {34.286, 31.429}}), Line(visible = true, origin = {48.571, -45.714}, rotation = -90, points = {{-45.714, -48.571}, {-15.714, -18.571}, {-15.714, 1.429}, {14.286, -8.571}, {4.286, 21.429}, {24.286, 21.429}, {34.286, 31.429}}), Line(visible = true, origin = {-90, 0}, points = {{0, 10}, {0, -10}}, thickness = 2), Line(visible = true, origin = {90, 90}, points = {{-10, 10}, {10, -10}}, thickness = 2), Line(visible = true, origin = {90, -90}, points = {{10, 10}, {-10, -10}}, thickness = 2), Line(visible = true, origin = {80, 80}, points = {{-10, 10}, {10, -10}}, thickness = 2), Line(visible = true, origin = {-80, 0}, points = {{0, 10}, {0, -10}}, thickness = 2), Line(visible = true, origin = {80, -80}, points = {{10, 10}, {-10, -10}}, thickness = 2), Line(visible = true, origin = {47.143, -2.857}, rotation = -180, points = {{-32.857, -2.857}, {-22.857, -2.857}, {-12.857, 17.143}, {-2.857, -22.857}, {7.143, 17.143}, {17.143, -2.857}, {47.143, -2.857}}), Line(visible = true, origin = {80, 0}, rotation = -180, points = {{0, 10}, {0, -10}}, thickness = 2), Line(visible = true, origin = {90, 0}, rotation = -180, points = {{0, 10}, {0, -10}}, thickness = 2), Line(visible = true, origin = {-45.714, -48.571}, rotation = -180, points = {{-45.714, -48.571}, {-15.714, -18.571}, {-15.714, 1.429}, {14.286, -8.571}, {4.286, 21.429}, {24.286, 21.429}, {34.286, 31.429}}), Line(visible = true, origin = {-48.571, 45.714}, rotation = -270, points = {{-45.714, -48.571}, {-15.714, -18.571}, {-15.714, 1.429}, {14.286, -8.571}, {4.286, 21.429}, {24.286, 21.429}, {34.286, 31.429}}), Line(visible = true, origin = {-80, -80}, points = {{-10, 10}, {10, -10}}, thickness = 2), Line(visible = true, origin = {-90, -90}, points = {{-10, 10}, {10, -10}}, thickness = 2), Line(visible = true, origin = {-82.299, 80}, points = {{10, 10}, {-10, -10}}, thickness = 2), Line(visible = true, origin = {-92.299, 90}, points = {{10, 10}, {-10, -10}}, thickness = 2), Text(visible = true, origin = {0, -230}, extent = {{-150, 70}, {150, 130}}, textString = "%name", fontName = "Arial")}), Diagram(coordinateSystem(extent = {{-148.5, -105.0}, {148.5, 105.0}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10})));
    end SixSpringReturn;
    annotation(Documentation(info = "<html></html>"), Icon(coordinateSystem(extent = {{-100.0, -100.0}, {100.0, 100.0}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Rectangle(visible = true, lineColor = {191, 191, 191}, fillColor = {255, 255, 255}, fillPattern = FillPattern.HorizontalCylinder, lineThickness = 4, extent = {{-90.0, -90.0}, {90.0, 90.0}}, radius = 25)}), Diagram(coordinateSystem(extent = {{-148.5, -105.0}, {148.5, 105.0}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10})));
  end Components;

  model JoystickForceResponse "Model simulating response of three joystick designs to circularly varying test force"
    extends Modelica.Icons.Example;
    inner Modelica.Mechanics.MultiBody.World world(nominalLength = 0.03, animateGravity = false, animateWorld = false) annotation(Placement(visible = true, transformation(origin = {-150, 60}, extent = {{10, -10}, {-10, 10}}, rotation = 180)));
    Joystick.Components.JoystickJoint joystickJoint1(d = 0.0005) annotation(Placement(visible = true, transformation(origin = {-110.0, -20.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Parts.BodyCylinder bodyCylinder1(r = {0, -0.1, 0}, density = 500, diameter = 0.01, color = {179, 53, 53}) annotation(Placement(visible = true, transformation(origin = {-80.0, 0.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = -90)));
    Modelica.Mechanics.MultiBody.Parts.BodyCylinder bodyCylinder2(r = {0, -0.02, 0}, density = 500, diameter = 0.005, color = {179, 53, 53}) annotation(Placement(visible = true, transformation(origin = {-80, -60}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
    Joystick.Components.RotatingRadialForce rotatingRadialForce2(f = appliedForce) annotation(Placement(visible = true, transformation(origin = {-80, 40}, extent = {{-10, -10}, {10, 10}}, rotation = -450)));
    Joystick.Components.ThreeSpringReturn threeSpringReturn(c = springConstants * 1, d = 0, s_unstretched = 0.02, spring_height = -0.02) annotation(Placement(visible = true, transformation(origin = {-80, -100}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation1(r = {0, -0.02, 0}) annotation(Placement(visible = true, transformation(origin = {-130, -60}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
    Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation2(r = {0, -0.02, 0}) annotation(Placement(visible = true, transformation(origin = {-30, -60}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
    Joystick.Components.FourSpringReturn fourSpringReturn(c = springConstants * 3 / 4, d = 0, s_unstretched = 0.02, spring_height = -0.02) annotation(Placement(visible = true, transformation(origin = {20, -100}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Joystick.Components.RotatingRadialForce rotatingRadialForce3(f = appliedForce) annotation(Placement(visible = true, transformation(origin = {20, 40}, extent = {{-10, -10}, {10, 10}}, rotation = -450)));
    Modelica.Mechanics.MultiBody.Parts.BodyCylinder bodyCylinder3(density = 500, diameter = 0.005, r = {0, -0.02, 0}, color = {126, 179, 53}) annotation(Placement(visible = true, transformation(origin = {20, -60}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
    Modelica.Mechanics.MultiBody.Parts.BodyCylinder bodyCylinder4(density = 500, diameter = 0.01, r = {0, -0.1, 0}, color = {126, 179, 53}) annotation(Placement(visible = true, transformation(origin = {20.0, 0.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = -90)));
    Joystick.Components.JoystickJoint joystickJoint2(d = 0.0005) annotation(Placement(visible = true, transformation(origin = {-10.0, -20.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation3(r = {0.1, 0, 0}, animation = false) annotation(Placement(visible = true, transformation(origin = {-60.0, 60.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = -360)));
    Joystick.Components.JoystickJoint joystickJoint3(d = 0.0005) annotation(Placement(visible = true, transformation(origin = {90.0, -20.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Parts.BodyCylinder bodyCylinder5(density = 500, diameter = 0.01, r = {0, -0.1, 0}, color = {53, 90, 179}) annotation(Placement(visible = true, transformation(origin = {120.0, 0.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = -90)));
    Modelica.Mechanics.MultiBody.Parts.BodyCylinder bodyCylinder6(density = 500, diameter = 0.005, r = {0, -0.02, 0}, color = {53, 90, 179}) annotation(Placement(visible = true, transformation(origin = {120, -60}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
    Joystick.Components.RotatingRadialForce rotatingRadialForce4(f = appliedForce) annotation(Placement(visible = true, transformation(origin = {120, 40}, extent = {{-10, -10}, {10, 10}}, rotation = -450)));
    Joystick.Components.SixSpringReturn sixSpringReturn(c = springConstants * 0.5, d = 0, s_unstretched = 0.02, spring_height = -0.02) annotation(Placement(visible = true, transformation(origin = {120, -100}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation4(r = {0, -0.02, 0}) annotation(Placement(visible = true, transformation(origin = {70, -60}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
    Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation5(r = {0.1, 0, 0}, animation = false) annotation(Placement(visible = true, transformation(origin = {40.0, 60.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = -360)));
    Modelica.Mechanics.MultiBody.Visualizers.FixedShape2 base(height(start = 0.4) = 0.4, length(start = 0.8) = 1, width(start = 0.002) = 0.002, lengthDirection = {1, 0, 0}, color = {255, 255, 255}, shapeType = "modelica://Joystick/DocumentationFiles/floor.obj", widthDirection = {0, 0, 1}, r_shape = {-3, -0.06, 0}) annotation(Placement(visible = true, transformation(origin = {110, 90}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Parts.Fixed fixed(r = {0, 0, 0}) annotation(Placement(visible = true, transformation(origin = {70, 90}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation6(r = {0, -0.05, 0}, width = 0.06) annotation(Placement(visible = true, transformation(origin = {-130, -120}, extent = {{-10, -10}, {10, 10}}, rotation = -810)));
    Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation7(r = {0, -0.05, 0}, width = 0.06) annotation(Placement(visible = true, transformation(origin = {-30, -121.504}, extent = {{-10, -10}, {10, 10}}, rotation = -810)));
    Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation8(r = {0, -0.05, 0}, width = 0.06) annotation(Placement(visible = true, transformation(origin = {70, -120}, extent = {{-10, -10}, {10, 10}}, rotation = -810)));
    parameter Modelica.Units.SI.Force appliedForce = 0.1 "Force applied on the joysticks";
    parameter Modelica.Units.SI.TranslationalSpringConstant springConstants = 100 "Base spring constant of the joystick springs";
  equation
    connect(base.frame_a, fixed.frame_b) annotation(Line(visible = true, origin = {90, 90}, points = {{10, 0}, {-10, 0}}, color = {95, 95, 95}));
    connect(fixedTranslation2.frame_b, fixedTranslation7.frame_a) annotation(Line(visible = true, origin = {-30, -90.752}, points = {{0, 20.752}, {-0, -20.752}}, color = {95, 95, 95}));
    connect(fixedTranslation4.frame_b, fixedTranslation8.frame_a) annotation(Line(visible = true, origin = {70, -90}, points = {{0, 20}, {0, -20}}, color = {95, 95, 95}));
    connect(fixedTranslation1.frame_b, fixedTranslation6.frame_a) annotation(Line(visible = true, origin = {-130, -90}, points = {{0, 20}, {0, -20}}, color = {95, 95, 95}));
    connect(bodyCylinder1.frame_a, rotatingRadialForce2.frame_b) annotation(Line(visible = true, origin = {-80, 20}, points = {{0, -10}, {0, 10}}, color = {95, 95, 95}));
    connect(joystickJoint1.frame_b, bodyCylinder1.frame_b) annotation(Line(visible = true, origin = {-86.633, -16.667}, points = {{-13.266, -3.333}, {6.633, -3.333}, {6.633, 6.667}}, color = {95, 95, 95}));
    connect(fixedTranslation4.frame_b, sixSpringReturn.frame_a) annotation(Line(visible = true, origin = {83.3, -90}, points = {{-13.3, 20}, {-13.3, -10}, {26.599, -10}}, color = {95, 95, 95}));
    connect(sixSpringReturn.frame_b, bodyCylinder6.frame_b) annotation(Line(visible = true, origin = {120, -85}, points = {{0, -15}, {0, 15}}, color = {95, 95, 95}));
    connect(fixedTranslation2.frame_b, fourSpringReturn.frame_a) annotation(Line(visible = true, origin = {-16.7, -90}, points = {{-13.3, 20}, {-13.3, -10}, {26.599, -10}}, color = {95, 95, 95}));
    connect(fourSpringReturn.frame_b, bodyCylinder3.frame_b) annotation(Line(visible = true, origin = {20, -85}, points = {{0, -15}, {0, 15}}, color = {95, 95, 95}));
    connect(fixedTranslation1.frame_b, threeSpringReturn.frame_a) annotation(Line(visible = true, origin = {-116.7, -90}, points = {{-13.3, 20}, {-13.3, -10}, {26.599, -10}}, color = {95, 95, 95}));
    connect(threeSpringReturn.frame_b, bodyCylinder2.frame_b) annotation(Line(visible = true, origin = {-80, -85}, points = {{0, -15}, {0, 15}}, color = {95, 95, 95}));
    connect(fixedTranslation3.frame_a, world.frame_b) annotation(Line(visible = true, origin = {-105, 60}, points = {{35, 0}, {-35, -0}}, color = {95, 95, 95}));
    connect(world.frame_b, joystickJoint1.frame_a) annotation(Line(visible = true, origin = {-132.25, 19.8}, points = {{-7.75, 40.2}, {2.25, 40.2}, {2.25, -40.2}, {12.25, -40.2}}, color = {95, 95, 95}));
    connect(joystickJoint3.frame_b, bodyCylinder6.frame_a) annotation(Line(visible = true, origin = {113.367, -30}, points = {{-13.266, 10}, {6.633, 10}, {6.633, -20}}, color = {95, 95, 95}));
    connect(joystickJoint3.frame_a, fixedTranslation4.frame_a) annotation(Line(visible = true, origin = {73.333, -30.267}, points = {{6.667, 9.867}, {-3.333, 9.867}, {-3.333, -19.733}}, color = {95, 95, 95}));
    connect(joystickJoint3.frame_b, bodyCylinder5.frame_b) annotation(Line(visible = true, origin = {113.367, -16.667}, points = {{-13.266, -3.333}, {6.633, -3.333}, {6.633, 6.667}}, color = {95, 95, 95}));
    connect(bodyCylinder5.frame_a, rotatingRadialForce4.frame_b) annotation(Line(visible = true, origin = {120, 20}, points = {{0, -10}, {0, 10}}, color = {95, 95, 95}));
    connect(fixedTranslation5.frame_b, fixedTranslation4.frame_a) annotation(Line(visible = true, origin = {63.333, 23.333}, points = {{-13.333, 36.667}, {6.667, 36.667}, {6.667, -73.333}}, color = {95, 95, 95}));
    connect(fixedTranslation3.frame_b, fixedTranslation5.frame_a) annotation(Line(visible = true, origin = {-10, 60}, points = {{-40, 0}, {40, 0}}, color = {95, 95, 95}));
    connect(bodyCylinder4.frame_a, rotatingRadialForce3.frame_b) annotation(Line(visible = true, origin = {20, 20}, points = {{0, -10}, {0, 10}}, color = {95, 95, 95}));
    connect(joystickJoint2.frame_a, fixedTranslation2.frame_a) annotation(Line(visible = true, origin = {-26.667, -30.267}, points = {{6.667, 9.867}, {-3.333, 9.867}, {-3.333, -19.733}}, color = {95, 95, 95}));
    connect(joystickJoint2.frame_b, bodyCylinder3.frame_a) annotation(Line(visible = true, origin = {13.367, -30}, points = {{-13.266, 10}, {6.633, 10}, {6.633, -20}}, color = {95, 95, 95}));
    connect(joystickJoint2.frame_b, bodyCylinder4.frame_b) annotation(Line(visible = true, origin = {13.367, -16.667}, points = {{-13.266, -3.333}, {6.633, -3.333}, {6.633, 6.667}}, color = {95, 95, 95}));
    connect(fixedTranslation3.frame_b, fixedTranslation2.frame_a) annotation(Line(visible = true, origin = {-36.667, 23.333}, points = {{-13.333, 36.667}, {6.667, 36.667}, {6.667, -73.333}}, color = {95, 95, 95}));
    connect(joystickJoint1.frame_b, bodyCylinder2.frame_a) annotation(Line(visible = true, origin = {-86.633, -30}, points = {{-13.266, 10}, {6.633, 10}, {6.633, -20}}, color = {95, 95, 95}));
    connect(joystickJoint1.frame_a, fixedTranslation1.frame_a) annotation(Line(visible = true, origin = {-126.667, -30.267}, points = {{6.667, 9.867}, {-3.333, 9.867}, {-3.333, -19.733}}, color = {95, 95, 95}));
    annotation(preferredView = "info", experiment(StopTime = 7), Diagram(coordinateSystem(extent = {{-170, -151.848}, {148.5, 105}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10})), __Wolfram(Animation(cameras = {Camera(name = "Default", center = {0.0899842080864062, -0.01812018026426539, -0.05017282418779712}, distance = 0.8335828736978279, rotation = {-0.2132216548554874, 0.1634126412371442, 0.04628692340974854, 0.962128034778037})}, windows = {Window(name = "Default", camera = "Default")}), ControlPanels(Panel(identifier = "fervent-hamilton", title = "Model parameters", elements = {Slider(variable = appliedForce, label = "Force applied on the joysticks", min = 0.05, max = 0.3), Slider(variable = springConstants, label = "Base spring constant", min = 50, max = 150)}))), Documentation(info = "<html>
<p>This model compares the response of three analog joysticks to a circularly varying test force. Each joystick has a handle re-centering mechanism based on a symmetrical arrangement of a different number of tension springs.</p>

<p>The other model in this example is <a href=\"modelica://Joystick.JoystickReturnTrajectory\">JoystickReturnTrajectory</a>, which compares the free return trajectory of the three joysticks.</p>

<h4>Hierarchical Modeling</h4>

<p>The joystick model is constructed hierarchically. Double-click on a component such as fourSpringReturn to see its model diagram. Inside fourSpringReturn double-click on one of the tension springs to see its model diagram, and so on:</p>

<p style=\"text-align: center;\"><img src=\"modelica://Joystick/DocumentationFiles/hierarchical.png\" style=\"width:569px\"  alt=\"hierarchical\" /></p>



<h4>Simulation &amp; Animation</h4>

<p>To simulate the model and view a 3D animation of it, follow the steps below:</p>


<p>Click the Simulate button:<img src=\"modelica://Joystick/DocumentationFiles/play.svg\" style=\"width:20px\" alt=\"simCenter\" /></p>

<p>Click the Animate button: <img src=\"modelica://Joystick/DocumentationFiles/animate.svg\" style=\"width:20px\" alt=\"animate\" /></p>
<p>Use your mouse or trackpad to drag the animation into a good angle, and zoom in with your scroll wheel or by using the trackpad. Then click the play button to play the animation.</p>


<p>The animation shows the response of the joystick handles to a test force that attempts to trace out a circle:</p>

<p style=\"text-align: center;\"><img src=\"modelica://Joystick/DocumentationFiles/animation.png\" style=\"width:569px\" style=\"width:550px\" alt=\"\" /></p>



<p>Note that the response of the joysticks is not perfectly circular. The joysticks with a larger number of symmetrically spaced springs (four or six) trace out a path that is more nearly circular.</p>

<h4>Visualization</h4>

<p>After simulating the model, look at the stored plots to see the path traced out by the joysticks.</p>


<p style=\"text-align: center;\"><img src=\"modelica://Joystick/DocumentationFiles/parametricPlotsForceResponse.png\" style=\"width:600px\" alt=\"\" /></p>

<p>You can see that the six-spring arrangement leads to the most circular force response.</p>

<p>By simulating the other model in this example, <a href=\"modelica://Joystick.JoystickReturnTrajectory\">JoystickReturnTrajectory</a>, you can also obtain the following parametric plots of the trajectory of each joystick as it re-centers itself after being displaced:</p>
<p style=\"text-align: center;\"><img src=\"modelica://Joystick/DocumentationFiles/parametricPlotsReturnTrajectory.png\" style=\"width:600px\" alt=\"animation\" /></p>



<p>You can see that the six-spring arrangement leads to the most linear return trajectory.</p>
</html>", figures = {Figure(title = "Joystick angles", identifier = "49262", preferred = true, plots = {Plot(curves = {Curve(x = joystickJoint1.revolute1.phi, y = joystickJoint1.revolute2.phi, legend = "Joystick 1 Angle")}, x = Axis(min = -20, max = 20, unit = "deg", label = "revolute1.phi"), y = Axis(min = -20, max = 20, unit = "deg", label = "revolute2.phi")), Plot(curves = {Curve(x = joystickJoint2.revolute1.phi, y = joystickJoint2.revolute2.phi, legend = "Joystick 2 Angle")}, x = Axis(min = -20, max = 20, unit = "deg", label = "revolute1.phi"), y = Axis(min = -20, max = 20, unit = "deg", label = "revolute2.phi")), Plot(curves = {Curve(x = joystickJoint3.revolute1.phi, y = joystickJoint3.revolute2.phi, legend = "Joystick 3 Angle")}, x = Axis(min = -20, max = 20, unit = "deg", label = "revolute1.phi"), y = Axis(min = -20, max = 20, unit = "deg", label = "revolute2.phi"))}), Figure(title = "Joystick XZ forces", identifier = "e5842", plots = {Plot(curves = {Curve(x = threeSpringReturn.frame_b.f[3], y = threeSpringReturn.frame_b.f[1], legend = "Joystick 1"), Curve(x = fourSpringReturn.frame_b.f[3], y = fourSpringReturn.frame_b.f[1], legend = "Joystick 2"), Curve(x = sixSpringReturn.frame_b.f[3], y = sixSpringReturn.frame_b.f[1], legend = "Joystick 3")})}), Figure(title = "Joystick Y forces", identifier = "9deb2", plots = {Plot(curves = {Curve(y = threeSpringReturn.frame_b.f[2], legend = "Joystick 1"), Curve(y = fourSpringReturn.frame_b.f[2], legend = "Joystick 2"), Curve(y = sixSpringReturn.frame_b.f[2], legend = "Joystick 3")})})}), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10})));
  end JoystickForceResponse;

  model JoystickReturnTrajectory "Model simulating response of three joystick designs to circularly varying test force"
    extends Modelica.Icons.Example;
    inner Modelica.Mechanics.MultiBody.World world(nominalLength = 0.03, animateGravity = false, animateWorld = false) annotation(Placement(visible = true, transformation(origin = {-150, 40}, extent = {{10, -10}, {-10, 10}}, rotation = 180)));
    Joystick.Components.JoystickJoint joystickJoint1(d = 0.0005, initialxangle = initialAngles[1], initialyangle = initialAngles[2]) annotation(Placement(visible = true, transformation(origin = {-110.0, -20.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Parts.BodyCylinder bodyCylinder1(r = {0, -0.1, 0}, density = 500, diameter = 0.01, color = {179, 53, 53}) annotation(Placement(visible = true, transformation(origin = {-80.0, 0.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = -90)));
    Modelica.Mechanics.MultiBody.Parts.BodyCylinder bodyCylinder2(r = {0, -0.02, 0}, density = 500, diameter = 0.005, color = {179, 53, 53}) annotation(Placement(visible = true, transformation(origin = {-80, -60}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
    Joystick.Components.ThreeSpringReturn threeSpringReturn(c = springConstants * 1, d = 0, s_unstretched = 0.02, spring_height = 0.02) annotation(Placement(visible = true, transformation(origin = {-80, -100}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation1(r = {0, -0.02, 0}) annotation(Placement(visible = true, transformation(origin = {-130, -60}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
    Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation2(r = {0, -0.02, 0}) annotation(Placement(visible = true, transformation(origin = {-30, -60}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
    Joystick.Components.FourSpringReturn fourSpringReturn(c = springConstants * 3 / 4, d = 0, s_unstretched = 0.02, spring_height = 0.02) annotation(Placement(visible = true, transformation(origin = {20, -100}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Parts.BodyCylinder bodyCylinder3(density = 500, diameter = 0.005, r = {0, -0.02, 0}, color = {126, 179, 53}) annotation(Placement(visible = true, transformation(origin = {20, -60}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
    Modelica.Mechanics.MultiBody.Parts.BodyCylinder bodyCylinder4(density = 500, diameter = 0.01, r = {0, -0.1, 0}, color = {126, 179, 53}) annotation(Placement(visible = true, transformation(origin = {20.0, 0.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = -90)));
    Joystick.Components.JoystickJoint joystickJoint2(d = 0.0005, initialxangle = initialAngles[1], initialyangle = initialAngles[2]) annotation(Placement(visible = true, transformation(origin = {-10.0, -20.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation3(r = {0.1, 0, 0}, animation = false) annotation(Placement(visible = true, transformation(origin = {-60, 40}, extent = {{-10, -10}, {10, 10}}, rotation = -360)));
    Joystick.Components.JoystickJoint joystickJoint3(d = 0.0005, initialxangle = initialAngles[1], initialyangle = initialAngles[2]) annotation(Placement(visible = true, transformation(origin = {90.0, -20.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Parts.BodyCylinder bodyCylinder5(density = 500, diameter = 0.01, r = {0, -0.1, 0}, color = {53, 90, 179}) annotation(Placement(visible = true, transformation(origin = {120.0, 0.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = -90)));
    Modelica.Mechanics.MultiBody.Parts.BodyCylinder bodyCylinder6(density = 500, diameter = 0.005, r = {0, -0.02, 0}, color = {53, 90, 179}) annotation(Placement(visible = true, transformation(origin = {120, -57.052}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
    Joystick.Components.SixSpringReturn sixSpringReturn(c = springConstants * 0.5, d = 0, s_unstretched = 0.02, spring_height = 0.02) annotation(Placement(visible = true, transformation(origin = {120, -100}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation4(r = {0, -0.02, 0}) annotation(Placement(visible = true, transformation(origin = {70, -60}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
    Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation5(r = {0.1, 0, 0}, animation = false) annotation(Placement(visible = true, transformation(origin = {40, 40}, extent = {{-10, -10}, {10, 10}}, rotation = -360)));
    Modelica.Mechanics.MultiBody.Visualizers.FixedShape2 base(height(start = 0.4) = 0.4, length(start = 0.8) = 1, width(start = 0.002) = 0.002, lengthDirection = {1, 0, 0}, color = {255, 255, 255}, shapeType = "modelica://Joystick/DocumentationFiles/floor.obj", widthDirection = {0, 0, 1}, r_shape = {-3, -0.06, 0}) annotation(Placement(visible = true, transformation(origin = {130, 80}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Parts.Fixed fixed(r = {0, 0, 0}) annotation(Placement(visible = true, transformation(origin = {90, 80}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation6(r = {0, -0.05, 0}, width = 0.06) annotation(Placement(visible = true, transformation(origin = {-130, -120}, extent = {{-10, -10}, {10, 10}}, rotation = -810)));
    Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation7(r = {0, -0.05, 0}, width = 0.06) annotation(Placement(visible = true, transformation(origin = {-30, -120}, extent = {{-10, -10}, {10, 10}}, rotation = -810)));
    Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation8(r = {0, -0.05, 0}, width = 0.06) annotation(Placement(visible = true, transformation(origin = {70, -120}, extent = {{-10, -10}, {10, 10}}, rotation = -810)));
    parameter Modelica.Units.SI.Angle initialAngles[2] = {0.5, 0.1} "Initial angles of the joysticks, first rotating aroung the X-axis and then rotating around the Y-axis";
    parameter Modelica.Units.SI.TranslationalSpringConstant springConstants = 100 "Base spring constant of the joystick springs";
  equation
    connect(base.frame_a, fixed.frame_b) annotation(Line(visible = true, origin = {110, 80}, points = {{10, 0}, {-10, 0}}, color = {95, 95, 95}));
    connect(fixedTranslation2.frame_b, fixedTranslation7.frame_a) annotation(Line(visible = true, origin = {-30, -90}, points = {{0, 20}, {0, -20}}, color = {95, 95, 95}));
    connect(fixedTranslation4.frame_b, fixedTranslation8.frame_a) annotation(Line(visible = true, origin = {70, -90}, points = {{0, 20}, {0, -20}}, color = {95, 95, 95}));
    connect(fixedTranslation1.frame_b, fixedTranslation6.frame_a) annotation(Line(visible = true, origin = {-130, -90}, points = {{0, 20}, {0, -20}}, color = {95, 95, 95}));
    connect(joystickJoint1.frame_b, bodyCylinder1.frame_b) annotation(Line(visible = true, origin = {-86.633, -16.667}, points = {{-13.266, -3.333}, {6.633, -3.333}, {6.633, 6.667}}, color = {95, 95, 95}));
    connect(fixedTranslation4.frame_b, sixSpringReturn.frame_a) annotation(Line(visible = true, origin = {83.3, -90}, points = {{-13.3, 20}, {-13.3, -10}, {26.599, -10}}, color = {95, 95, 95}));
    connect(sixSpringReturn.frame_b, bodyCylinder6.frame_b) annotation(Line(visible = true, origin = {120, -83.526}, points = {{0, -16.474}, {0, 16.474}}, color = {95, 95, 95}));
    connect(fixedTranslation2.frame_b, fourSpringReturn.frame_a) annotation(Line(visible = true, origin = {-16.7, -90}, points = {{-13.3, 20}, {-13.3, -10}, {26.599, -10}}, color = {95, 95, 95}));
    connect(fourSpringReturn.frame_b, bodyCylinder3.frame_b) annotation(Line(visible = true, origin = {20, -85}, points = {{0, -15}, {0, 15}}, color = {95, 95, 95}));
    connect(fixedTranslation1.frame_b, threeSpringReturn.frame_a) annotation(Line(visible = true, origin = {-116.7, -90}, points = {{-13.3, 20}, {-13.3, -10}, {26.599, -10}}, color = {95, 95, 95}));
    connect(threeSpringReturn.frame_b, bodyCylinder2.frame_b) annotation(Line(visible = true, origin = {-80, -85}, points = {{0, -15}, {0, 15}}, color = {95, 95, 95}));
    connect(fixedTranslation3.frame_a, world.frame_b) annotation(Line(visible = true, origin = {-105, 40}, points = {{35, 0}, {-35, 0}}, color = {95, 95, 95}));
    connect(world.frame_b, joystickJoint1.frame_a) annotation(Line(visible = true, origin = {-132.043, 9.9}, points = {{-7.957, 30.1}, {2.043, 30.1}, {2.043, -29.9}, {12.043, -30.3}}, color = {95, 95, 95}));
    connect(joystickJoint3.frame_b, bodyCylinder6.frame_a) annotation(Line(visible = true, origin = {113.367, -29.017}, points = {{-13.266, 9.017}, {6.633, 9.017}, {6.633, -18.035}}, color = {95, 95, 95}));
    connect(joystickJoint3.frame_a, fixedTranslation4.frame_a) annotation(Line(visible = true, origin = {73.333, -30.267}, points = {{6.667, 9.867}, {-3.333, 9.867}, {-3.333, -19.733}}, color = {95, 95, 95}));
    connect(joystickJoint3.frame_b, bodyCylinder5.frame_b) annotation(Line(visible = true, origin = {113.367, -16.667}, points = {{-13.266, -3.333}, {6.633, -3.333}, {6.633, 6.667}}, color = {95, 95, 95}));
    connect(fixedTranslation5.frame_b, fixedTranslation4.frame_a) annotation(Line(visible = true, origin = {63.333, 10}, points = {{-13.333, 30}, {6.667, 30}, {6.667, -60}}, color = {95, 95, 95}));
    connect(fixedTranslation3.frame_b, fixedTranslation5.frame_a) annotation(Line(visible = true, origin = {-10, 40}, points = {{-40, 0}, {40, 0}}, color = {95, 95, 95}));
    connect(joystickJoint2.frame_a, fixedTranslation2.frame_a) annotation(Line(visible = true, origin = {-26.667, -30.267}, points = {{6.667, 9.867}, {-3.333, 9.867}, {-3.333, -19.733}}, color = {95, 95, 95}));
    connect(joystickJoint2.frame_b, bodyCylinder3.frame_a) annotation(Line(visible = true, origin = {13.367, -30}, points = {{-13.266, 10}, {6.633, 10}, {6.633, -20}}, color = {95, 95, 95}));
    connect(joystickJoint2.frame_b, bodyCylinder4.frame_b) annotation(Line(visible = true, origin = {13.367, -16.667}, points = {{-13.266, -3.333}, {6.633, -3.333}, {6.633, 6.667}}, color = {95, 95, 95}));
    connect(fixedTranslation3.frame_b, fixedTranslation2.frame_a) annotation(Line(visible = true, origin = {-36.667, 10}, points = {{-13.333, 30}, {6.667, 30}, {6.667, -60}}, color = {95, 95, 95}));
    connect(joystickJoint1.frame_b, bodyCylinder2.frame_a) annotation(Line(visible = true, origin = {-86.633, -30}, points = {{-13.266, 10}, {6.633, 10}, {6.633, -20}}, color = {95, 95, 95}));
    connect(joystickJoint1.frame_a, fixedTranslation1.frame_a) annotation(Line(visible = true, origin = {-126.667, -30.267}, points = {{6.667, 9.867}, {-3.333, 9.867}, {-3.333, -19.733}}, color = {95, 95, 95}));
    annotation(preferredView = "info", experiment(StopTime = 1, __Wolfram_SynchronizeWithRealTime = false), Diagram(coordinateSystem(extent = {{-170, -156.558}, {148.5, 105}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10})), __Wolfram(Animation(cameras = {Camera(name = "Default", center = {0.06750845989045581, -0.06068596493872246, -0.01351625646386112}, distance = 0.9169412196963997, rotation = {-0.2428837448772426, 0.1048988874327984, 0.0514217976646295, 0.9629950719576428})}, windows = {Window(name = "Default", camera = "Default")}), ControlPanels(Panel(identifier = "unruffled-laplace", title = "Model parameters", elements = {Slider(variable = springConstants, label = "Base spring constant", min = 1, max = 150)}))), Documentation(info = "<html>
<p>This model compares the return trajectory of three analog joysticks after being suddenly released after being displaced from the center equilibrium position. Each joystick has a handle re-centering mechanism based on a symmetrical arrangement of a different number of tension springs.
</p>

<p>The other model in this example is <a href=\"modelica://Joystick.JoystickForceResponse\">JoystickForceResponse</a>, which compares the response of the three joysticks to a circularly varying test force.</p>

<h4>Hierarchical Modeling</h4>

<p>The joystick model is constructed hierarchically. Double-click on a component such as fourSpringReturn to see its model diagram. Inside fourSpringReturn double-click on one of the tension springs to see its model diagram, and so on:</p>



<p style=\"text-align: center;\"><img src=\"modelica://Joystick/DocumentationFiles/hierarchical.png\" style=\"width:569px\" alt=\"hierarchical\" /></p>

<h4>Simulation &amp; Animation</h4>

<p>To simulate the model and view a 3D animation of it, follow the steps below:</p>


<p>Click the Simulate button:<img src=\"modelica://Joystick/DocumentationFiles/play.svg\" style=\"width:20px\" alt=\"simCenter\" /></p>

<p>Click the Animate button: <img src=\"modelica://Joystick/DocumentationFiles/animate.svg\" style=\"width:20px\" alt=\"animate\" /></p>
<p>Use your mouse or trackpad to drag the animation into a good angle, and zoom in with your scroll wheel or by using the trackpad. Then click the play button to play the animation.</p>
<p>Change the Time Scale at the bottom of the animation window to 0.1 to watch the joystick in slow motion.</p>


<p>The animation shows the trajectory of the joystick handle as it springs back after being suddenly released from a position away from the center equilibrium:</p>



<p style=\"text-align: center;\"><img src=\"modelica://Joystick/DocumentationFiles/animation.png\" style=\"width:550px\" alt=\"\" /></p>

<p>Note that the handle does not return along a linear trajectory. The joysticks with a larger number of symmetrically spaced springs (four or six) follow a path that is more nearly linear.</p>

<h4>Visualization</h4>

<p>After simulating the model, look at the stored plots to see parametric plots of the path along which each joystick returns.</p>

<p>The parametric plots look like this:</p>


<p style=\"text-align: center;\"><img src=\"modelica://Joystick/DocumentationFiles/parametricPlotsReturnTrajectory.png\" style=\"width:600px\" alt=\"hierarchical\" /></p>


<p>You can see that the six-spring arrangement leads to the most linear return trajectory.</p>

<p>By simulating the other model in this example, <a href=\"modelica://Joystick.JoystickForceResponse\">JoystickForceResponse</a>, you can also obtain the following parametric plots of the response of each joystick to a circularly varying test force:</p>



<p style=\"text-align: center;\"><img src=\"modelica://Joystick/DocumentationFiles/parametricPlotsForceResponse.png\" style=\"width:600px\" alt=\"hierarchical\" /></p>

<p>You can see that the-six spring arrangement leads to the most circular force response.</p>

</html>", figures = {Figure(title = "Joystick angles", identifier = "05d4d", preferred = true, plots = {Plot(curves = {Curve(x = joystickJoint1.revolute1.phi, y = joystickJoint1.revolute2.phi, legend = "Joystick 1 Angle")}, x = Axis(min = -10, max = 30, unit = "deg", label = "revolute1.phi"), y = Axis(min = -5, max = 10, unit = "deg", label = "revolute2.phi")), Plot(curves = {Curve(x = joystickJoint2.revolute1.phi, y = joystickJoint2.revolute2.phi, legend = "Joystick 2 Angle")}, x = Axis(min = -10, max = 30, unit = "deg", label = "revolute1.phi"), y = Axis(min = -5, max = 10, unit = "deg", label = "revolute2.phi")), Plot(curves = {Curve(x = joystickJoint3.revolute1.phi, y = joystickJoint3.revolute2.phi, legend = "Joystick 3 Angle")}, x = Axis(min = -10, max = 30, unit = "deg", label = "revolute1.phi"), y = Axis(min = -5, max = 10, unit = "deg", label = "revolute2.phi"))}), Figure(title = "Joystick XZ forces", identifier = "e3501", plots = {Plot(curves = {Curve(x = threeSpringReturn.frame_b.f[3], y = threeSpringReturn.frame_b.f[1], legend = "Joystick 1"), Curve(x = fourSpringReturn.frame_b.f[3], y = fourSpringReturn.frame_b.f[1], legend = "Joystick 2"), Curve(x = sixSpringReturn.frame_b.f[3], y = sixSpringReturn.frame_b.f[1], legend = "Joystick 3")})}), Figure(title = "Joystick Y forces", identifier = "cd75a", plots = {Plot(curves = {Curve(y = threeSpringReturn.frame_b.f[2], legend = "Joystick 1"), Curve(y = fourSpringReturn.frame_b.f[2], legend = "Joystick 2"), Curve(y = sixSpringReturn.frame_b.f[2], legend = "Joystick 3")})})}), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10})));
  end JoystickReturnTrajectory;
  annotation(preferredView = "info", uses(Modelica(version = "4.0.0")), Diagram(coordinateSystem(extent = {{-148.5, -105.0}, {148.5, 105.0}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10})), Documentation(info = "<html></html>"), __Wolfram(openClass = Joystick.JoystickForceResponse), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Rectangle(visible = true, lineColor = {111, 0, 0}, fillColor = {255, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, lineThickness = 4, extent = {{-90, -90}, {90, 90}}, radius = 25), Text(visible = true, origin = {12, 20.833}, textColor = {255, 255, 255}, extent = {{-102, -70.833}, {78, 29.167}}, textString = "Ex", fontName = "Arial")}));
end Joystick;
